'From Squeak3.2gamma of 15 January 2002 [latest update: #4743] on 25 July 2002 at 11:25:24 pm'!
ProtoObject subclass: #MaMinimalObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma base classes-Abstractions'!
MaMinimalObject subclass: #MaMutatingProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Proxys'!

!MaMinimalObject methodsFor: 'Public methods'!
= anObject 
	"Answer whether the receiver and the argument represent the same 
	object. If = is redefined in any subclass, consider also redefining the 
	message hash."

	^self == anObject! !

!MaMinimalObject methodsFor: 'Public methods'!
basicSize
	"Primitive. Answer the number of indexable variables in the receiver. 
	This value is the same as the largest legal subscript. Essential. Do not 
	override in any subclass. See Object documentation whatIsAPrimitive."

	<primitive: 62>
	"The number of indexable fields of fixed-length objects is 0"
	^0 ! !

!MaMinimalObject methodsFor: 'Public methods'!
~= anObject

	"Answer a Boolean which is false when the receiver and anObject
	 are equivalent, and true otherwise."

	^(self = anObject) not! !

!MaMinimalObject methodsFor: 'accessing'!
basicClass

	^self proxyClass
! !

!MaMinimalObject methodsFor: 'accessing'!
class

	^self proxyClass
! !

!MaMinimalObject methodsFor: 'accessing'!
proxyClass

	"Answer a Class which is the receiver's class"

	<primitive: 111>
	self primitiveFailed
! !

!MaMinimalObject methodsFor: 'accessing'!
yourself

	^self
! !

!MaMinimalObject methodsFor: 'actions'!
error: aString

	"Stop the currently executing process and display
	 the error message aString."

	^nil error: aString
! !

!MaMinimalObject methodsFor: 'actions'!
finalize

	^self
! !

!MaMinimalObject methodsFor: 'actions'!
halt
	"This is the typical message to use for inserting breakpoints during 
	debugging. It behaves like halt:, but does not call on halt: in order to 
	avoid putting this message on the stack. Halt is especially useful when 
	the breakpoint message is an arbitrary one."

	Halt signal
! !

!MaMinimalObject methodsFor: 'actions'!
inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."

	Inspector openOn: self withEvalPane: true
! !

!MaMinimalObject methodsFor: 'actions'!
shallowCopy
	"Answer a copy of the receiver which shares the receiver's instance variables."
	| class newObject index |
	<primitive: 148>
	class _ self class.
	class isVariable
		ifTrue: 
			[index _ self basicSize.
			newObject _ class basicNew: index.
			[index > 0]
				whileTrue: 
					[newObject basicAt: index put: (self basicAt: index).
					index _ index - 1]]
		ifFalse: [newObject _ class basicNew].
	index _ class instSize.
	[index > 0]
		whileTrue: 
			[newObject instVarAt: index put: (self instVarAt: index).
			index _ index - 1].
	^ newObject
! !

!MaMinimalObject methodsFor: 'debugging'!
halt: aString 
	"This is the typical message to use for inserting breakpoints during 
	debugging. It creates and schedules a Notifier with the argument, 
	aString, as the label."
	
	Halt new signal: aString
! !

!MaMinimalObject methodsFor: 'debugging'!
instVarAt: index 
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 73>
	"Access beyond fixed variables."
	^self basicAt: index - self class instSize
! !

!MaMinimalObject methodsFor: 'testing'!
maIsMinimalObject

	^true
! !

!MaMinimalObject methodsFor: 'printing'!
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."

	| title |
	title _ self class name.
	aStream
		nextPutAll: (title first isVowel ifTrue: ['an '] ifFalse: ['a ']);
		nextPutAll: title
! !

!MaMinimalObject methodsFor: 'printing'!
printString
	"Answer a String whose characters are a description of the receiver. 
	If you want to print without a character limit, use fullPrintString."

	^ self printStringLimitedTo: 50000
! !


!MaMutatingProxy methodsFor: 'private actions' stamp: 'cmm 5/15/2002 21:46'!
becomeForward: otherObject 
	"Primitive. All variables in the entire system that used to point
	to the receiver now point to the argument.
	Fails if either argument is a SmallInteger."

	(Array with: self)
		elementsForwardIdentityTo:
			(Array with: otherObject)
! !

!MaMutatingProxy methodsFor: 'private actions'!
doesNotUnderstand: aMessage

	^self mutateAndSend: aMessage
! !

!MaMutatingProxy methodsFor: 'private actions'!
mutateAndSend: aMessage

	^self mutant
		perform: aMessage selector
		withArguments: aMessage arguments
! !

!MaMutatingProxy methodsFor: 'private creation'!
mutant

	self subclassResponsibility
! !


!Object methodsFor: 'testing'!
maIsMinimalObject

	^false
! !


!MaMinimalObject reorganize!
('Public methods' = basicSize ~=)
('accessing' basicClass class proxyClass yourself)
('actions' error: finalize halt inspect shallowCopy)
('debugging' halt: instVarAt:)
('testing' maIsBufferedObjectProxy maIsImmutableInBuffer maIsMinimalObject)
('printing' printOn: printString)
!

